﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
// Only using the general Ivi Assemblies, no manufacturer-specific reference is used.
// This allows for the code to be interchangeable.
using Ivi.Driver;
using Ivi.SpecAn;

// Preconditions:
// - Ivi Shared Components 2.42 or newer
// - Ivi .NET Shared Components 1.2.0 or newer
// - RsSpecan IVI.NET instrument driver 3.8.0 or newer
//
// Ivi Configuration Store:
// - Define new Driver Session in your  with:
//    - Hardware Asset fitting your physical instrument: R&S FSW / FSV / FPS / FSWP
//    - Software Module = Rohde&Schwarz.RsSpecAn.Fx40
//
// - Define new Logical Name "FSW" that you will use in the IviSpecAn.Create() method
// - Link this Logical Name to your previously defined Driver Session.
// - Run the example.

namespace IviSpecAn_WaveformMeasurement
{
    class Program
    {
        static void Main(string[] args)
        {
            IIviSpecAn specan;

            try //separate try-catch for specan initialization prevents accessing uninitialized object
            {
                specan = IviSpecAn.Create("FSW");
                Console.WriteLine("Driver component: {0} {1} {2}\n", specan.Identity.Vendor, specan.Identity.Identifier, specan.Identity.Revision);
            }
            catch (Ivi.Driver.DriverClassCreationException e)
            {
                Console.WriteLine("Error initializing the specan session:\n{0}", e.Message);
                Console.WriteLine("Press any key to finish.");
                Console.ReadKey();
                return;
            }
            try
            {
                specan.Utility.Reset();

                // Configure the Acquisition
                specan.Acquisition.SweepModeContinuous = false;
                specan.Acquisition.NumberOfSweeps = 10;
                specan.Acquisition.DetectorType = DetectorType.Rms;
                specan.Acquisition.VerticalScale = VerticalScale.Linear;

                specan.Frequency.ConfigureCenterSpan(1.1E9, 200E6);
                specan.Level.Reference = -15;
                specan.Level.Attenuation = 50;
                specan.Level.AmplitudeUnits = AmplitudeUnits.dBm;

                specan.SweepCoupling.ResolutionBandwidth = 100E3;
                specan.SweepCoupling.VideoBandwidth = 300E3;
                specan.SweepCoupling.SweepTime = new PrecisionTimeSpan(0, 0.5);

                // Configure 2 traces for maxhold and minhold
                specan.Traces["TR1"].Type = TraceType.MaxHold;
                specan.Traces["TR2"].Type = TraceType.MinHold;

                // Initiate the acquisition and wait for it to finish
                specan.Traces.Initiate();

                // Fetch the Trace 1, no new sweep is being performed
                ISpectrum<double> spectrumTrace1 = null;
                spectrumTrace1 = specan.Traces["TR1"].FetchY(spectrumTrace1);
                Console.WriteLine("Trace1 acquired {0} sweeppoints, Start frequency {1:F3} MHz, Stop frequency {2:F3} MHz",
                                    spectrumTrace1.ValidPointCount, spectrumTrace1.StartFrequency / 1E6, spectrumTrace1.StopFrequency/1E6);
                
                // Fetch the Trace 2, no new sweep is being performed
                ISpectrum<double> spectrumTrace2 = null;
                spectrumTrace2 = specan.Traces["TR2"].FetchY(spectrumTrace2);
                Console.WriteLine("Trace2 acquired {0} sweeppoints, Start frequency {1:F3} MHz, Stop frequency {2:F3} MHz",
                                    spectrumTrace2.ValidPointCount, spectrumTrace2.StartFrequency / 1E6, spectrumTrace2.StopFrequency/1E6);

                // Set Marker 1 to Trace 2 and search max peak
                specan.Marker.ActiveMarker = "M1";
                specan.Marker.Trace = "2";
                specan.Marker.Enabled = true;
                specan.Marker.ConfigureSearch(3.0, -150.0);
                specan.Marker.Search(MarkerSearch.Highest);
                var marker1 = specan.Marker.Query();
                Console.WriteLine("Marker 1 for Trace 2 max: Frequency {0:F3} MHz, Level {1:F2} dBm", marker1.Position/1E6, marker1.Amplitude);

                // Set Marker 2 to Trace 1 and search max peak
                specan.Marker.ActiveMarker = "M2";
                specan.Marker.Trace = "1";
                specan.Marker.Enabled = true;
                specan.Marker.ConfigureSearch(1.0, -110.0);
                specan.Marker.Search(MarkerSearch.Highest);
                var marker2 = specan.Marker.Query();
                Console.WriteLine("Marker 2 for Trace 1 max: Frequency {0:F3} MHz, Level {1:F2} dBm", marker2.Position / 1E6, marker2.Amplitude);
            }

            catch (Ivi.Driver.InstrumentStatusException e)
            {
                Console.WriteLine("Instrument status exception:\n{0}", e.Message);
            }

            finally
            {
                Console.WriteLine("\nPress any key to finish.");
                Console.ReadKey();
            }
        }
    }
}
